
#include <cstdlib>
#include <memory>

#include <fggl/app.hpp>
#include <fggl/ecs3/types.hpp>
#include <fggl/ecs3/module/module.h>
#include <fggl/util/service.h>

namespace fggl {

    App::App(const Identifer& name) : App::App( name, name ) {
    }

    App::App(const Identifer& name, const Identifer& folder ) :
        m_running(true),
        m_types(std::make_unique<ecs3::TypeRegistry>()),
        m_modules(std::make_unique<ecs3::ModuleManager>(*m_types)),
        m_states() {
    }

    int App::run(int argc, const char** argv) {

        {
            // activate the first state
            auto& state = m_states.active();
            state.activate();
        }

        while ( m_running ) {
            auto& state = m_states.active();

            state.update();
            state.render();
        }

        {
            // shutdown last state
            auto& state = m_states.active();
            state.deactivate();
        }

        return EXIT_SUCCESS;
    }

} //namespace fggl

